/**
	Classes definiton for the
	Knight Chess Problem

	Fraga 05/11/2000
**/

#ifndef KNIGHT_H
#define KNIGHT_H


#include <qpushbutton.h>
#include <qvector.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qpixmap.h>


// --------------------------------------------------------------------------
// ChessButton implements a single button
//

class ChessButton : public QButton
{
    Q_OBJECT
public:
    ChessButton( QWidget *parent=0 );
    enum Type { Blank, Passed, Actual };
    enum Color{ White, Black };
    Type	type() const		{ return t; }
    void	setType( Type type )	{ t = type; repaint(); }
    void	setColor( Color color )	{ c = color; }
    void	setValue( int v )	{ val = v; }
	QPixmap *img;
protected:
	void	drawButton( QPainter * );
private:
    Type t;
	Color c;
	int val;  // Draw this value if it is in !Blank state
};

// Using template vector to make vector-class of ChessButton.
// This vector is used by the GameBoard class defined below.

typedef QVector<ChessButton>	ChessButtons;
// typedef QArray<int>		ChessArray;


// --------------------------------------------------------------------------
// ChessGameBoard implements the chess game board.
// ChessGameBoard is a composite widget that contains 8x8 SquareButtons.
//

class ChessGameBoard : public QWidget
{
    Q_OBJECT
public:
    ChessGameBoard( int n, QPixmap *pixmap=0, 
			QWidget *parent=0, const char *name=0 );
   ~ChessGameBoard();
    enum	State { Init, Playing, Gameover, Badmove };
    State	state() const		{ return st; }
    void	computerStarts( bool v );
    void    newGame();
	int		getPosRow( ) { return irow; } 
	int		getPosCol( ) { return icol; } 
	int		getCounter( ) { return counter; } 
	void	getNewPos( int *row, int *col ) { *row=nrow; *col=ncol; } 
	void 	ChangeButtons( int newrow, int newcol );
	void    PlayGame ( );
	bool    WhoPlay ( ) { return comp_starts; }
	bool	check_next_trial( int row, int col );
	bool	test_movement( int row, int col );
	void 	TryMove( int row, int col );
//
signals:
    void	finished();			// game finished
    void	newPosition();		// change position
    void	newMessagePosition();	// change message position
private slots:
    void	buttonClicked();
protected:
	void timerEvent( QTimerEvent  * );
private:
	void	setIniPos( int row, int col ) { irow=row; icol=col; } 
	void	setNewPos( int row, int col ) { nrow=row; ncol=col; } 
	void	setCounter( int i ) { counter=i; } 
    void    setState( State state ) { st = state; }
    void    calculate_accessboard( );
//
	int trymove( int y, int x, int *to_y, int *to_x);
    void    computerPlay( );
//
    State	st;
    int		nBoard;
	int  irow, icol;   // Initial coordinates
	int  nrow, ncol;   // These will be the new coordinates
	int counter; // Number of hits. Maximun are 64
    bool	comp_starts;
    ChessButtons *buttons;
	bool playing;
//
	// Variables for the game
	int h[8][8];
	int accessibility[8];
	// Constants for the game
	int aa[8], bb[8];
	int accessboard[8][8];
};



// --------------------------------------------------------------------------
// ChessGame implements the complete game.
// ChessGame is a composite widget that contains a ChessGameBoard and
// two push buttons for starting the game and quitting.
//

class ChessGame : public QWidget
{
    Q_OBJECT
public:
    ChessGame( int boardSize=8, QPixmap *pixmap=0, 
	 		QWidget *parent=0, const char *name=0 );
	void HumanPlay( int row, int col );
private slots:
    void	startGameClicked();
    void	newGameClicked();
    void	gameOver();
    void	changePosition();
    void	spin1Clicked();
    void	spin2Clicked();
    void	setMessPoss();
private:
	int flagsboxes;
    void	newState();
    QLabel	*message;
    QLabel	*messposs;
    QComboBox	*whoPlay;
    QPushButton *startGame;
    QPushButton *newGame;
    QPushButton *quit;
	QSpinBox *sbox1;
	QSpinBox *sbox2;
    ChessGameBoard *board;
};


#endif // KNIGHT_H
